from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class InterfacePayload(TypedDict, total=False):
    """
    Type hints for system/interface payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: InterfacePayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: NotRequired[str]  # Name.
    vdom: str  # Interface is in this virtual domain (VDOM).
    vrf: NotRequired[int]  # Virtual Routing Forwarding ID.
    cli_conn_status: NotRequired[int]  # CLI connection status.
    fortilink: NotRequired[Literal["enable", "disable"]]  # Enable FortiLink to dedicate this interface to manage other 
    switch_controller_source_ip: NotRequired[Literal["outbound", "fixed"]]  # Source IP address used in FortiLink over L3 connections.
    mode: NotRequired[Literal["static", "dhcp", "pppoe"]]  # Addressing mode (static, DHCP, PPPoE).
    client_options: NotRequired[list[dict[str, Any]]]  # DHCP client options.
    distance: NotRequired[int]  # Distance for routes learned through PPPoE or DHCP, lower dis
    priority: NotRequired[int]  # Priority of learned routes.
    dhcp_relay_interface_select_method: NotRequired[Literal["auto", "sdwan", "specify"]]  # Specify how to select outgoing interface to reach server.
    dhcp_relay_interface: str  # Specify outgoing interface to reach server.
    dhcp_relay_vrf_select: NotRequired[int]  # VRF ID used for connection to server.
    dhcp_broadcast_flag: NotRequired[Literal["disable", "enable"]]  # Enable/disable setting of the broadcast flag in messages sen
    dhcp_relay_service: NotRequired[Literal["disable", "enable"]]  # Enable/disable allowing this interface to act as a DHCP rela
    dhcp_relay_ip: NotRequired[str]  # DHCP relay IP address.
    dhcp_relay_source_ip: NotRequired[str]  # IP address used by the DHCP relay as its source IP.
    dhcp_relay_circuit_id: NotRequired[str]  # DHCP relay circuit ID.
    dhcp_relay_link_selection: NotRequired[str]  # DHCP relay link selection.
    dhcp_relay_request_all_server: NotRequired[Literal["disable", "enable"]]  # Enable/disable sending of DHCP requests to all servers.
    dhcp_relay_allow_no_end_option: NotRequired[Literal["disable", "enable"]]  # Enable/disable relaying DHCP messages with no end option.
    dhcp_relay_type: NotRequired[Literal["regular", "ipsec"]]  # DHCP relay type (regular or IPsec).
    dhcp_smart_relay: NotRequired[Literal["disable", "enable"]]  # Enable/disable DHCP smart relay.
    dhcp_relay_agent_option: NotRequired[Literal["enable", "disable"]]  # Enable/disable DHCP relay agent option.
    dhcp_classless_route_addition: NotRequired[Literal["enable", "disable"]]  # Enable/disable addition of classless static routes retrieved
    management_ip: NotRequired[str]  # High Availability in-band management IP address of this inte
    ip: NotRequired[str]  # Interface IPv4 address and subnet mask, syntax: X.X.X.X/24.
    allowaccess: NotRequired[Literal["ping", "https", "ssh", "snmp", "http", "telnet", "fgfm", "radius-acct", "probe-response", "fabric", "ftm", "speed-test", "scim"]]  # Permitted types of management access to this interface.
    gwdetect: NotRequired[Literal["enable", "disable"]]  # Enable/disable detect gateway alive for first.
    ping_serv_status: NotRequired[int]  # PING server status.
    detectserver: NotRequired[str]  # Gateway's ping server for this IP.
    detectprotocol: NotRequired[Literal["ping", "tcp-echo", "udp-echo"]]  # Protocols used to detect the server.
    ha_priority: NotRequired[int]  # HA election priority for the PING server.
    fail_detect: NotRequired[Literal["enable", "disable"]]  # Enable/disable fail detection features for this interface.
    fail_detect_option: NotRequired[Literal["detectserver", "link-down"]]  # Options for detecting that this interface has failed.
    fail_alert_method: NotRequired[Literal["link-failed-signal", "link-down"]]  # Select link-failed-signal or link-down method to alert about
    fail_action_on_extender: NotRequired[Literal["soft-restart", "hard-restart", "reboot"]]  # Action on FortiExtender when interface fail.
    fail_alert_interfaces: NotRequired[list[dict[str, Any]]]  # Names of the FortiGate interfaces to which the link failure 
    dhcp_client_identifier: NotRequired[str]  # DHCP client identifier.
    dhcp_renew_time: NotRequired[int]  # DHCP renew time in seconds (300-604800), 0 means use the ren
    ipunnumbered: NotRequired[str]  # Unnumbered IP used for PPPoE interfaces for which no unique 
    username: NotRequired[str]  # Username of the PPPoE account, provided by your ISP.
    pppoe_egress_cos: NotRequired[Literal["cos0", "cos1", "cos2", "cos3", "cos4", "cos5", "cos6", "cos7"]]  # CoS in VLAN tag for outgoing PPPoE/PPP packets.
    pppoe_unnumbered_negotiate: NotRequired[Literal["enable", "disable"]]  # Enable/disable PPPoE unnumbered negotiation.
    password: NotRequired[str]  # PPPoE account's password.
    idle_timeout: NotRequired[int]  # PPPoE auto disconnect after idle timeout seconds, 0 means no
    detected_peer_mtu: NotRequired[int]  # MTU of detected peer (0 - 4294967295).
    disc_retry_timeout: NotRequired[int]  # Time in seconds to wait before retrying to start a PPPoE dis
    padt_retry_timeout: NotRequired[int]  # PPPoE Active Discovery Terminate (PADT) used to terminate se
    service_name: NotRequired[str]  # PPPoE service name.
    ac_name: NotRequired[str]  # PPPoE server name.
    lcp_echo_interval: NotRequired[int]  # Time in seconds between PPPoE Link Control Protocol (LCP) ec
    lcp_max_echo_fails: NotRequired[int]  # Maximum missed LCP echo messages before disconnect.
    defaultgw: NotRequired[Literal["enable", "disable"]]  # Enable to get the gateway IP from the DHCP or PPPoE server.
    dns_server_override: NotRequired[Literal["enable", "disable"]]  # Enable/disable use DNS acquired by DHCP or PPPoE.
    dns_server_protocol: NotRequired[Literal["cleartext", "dot", "doh"]]  # DNS transport protocols.
    auth_type: NotRequired[Literal["auto", "pap", "chap", "mschapv1", "mschapv2"]]  # PPP authentication type to use.
    pptp_client: NotRequired[Literal["enable", "disable"]]  # Enable/disable PPTP client.
    pptp_user: NotRequired[str]  # PPTP user name.
    pptp_password: NotRequired[str]  # PPTP password.
    pptp_server_ip: NotRequired[str]  # PPTP server IP address.
    pptp_auth_type: NotRequired[Literal["auto", "pap", "chap", "mschapv1", "mschapv2"]]  # PPTP authentication type.
    pptp_timeout: NotRequired[int]  # Idle timer in minutes (0 for disabled).
    arpforward: NotRequired[Literal["enable", "disable"]]  # Enable/disable ARP forwarding.
    ndiscforward: NotRequired[Literal["enable", "disable"]]  # Enable/disable NDISC forwarding.
    broadcast_forward: NotRequired[Literal["enable", "disable"]]  # Enable/disable broadcast forwarding.
    bfd: NotRequired[Literal["global", "enable", "disable"]]  # Bidirectional Forwarding Detection (BFD) settings.
    bfd_desired_min_tx: NotRequired[int]  # BFD desired minimal transmit interval.
    bfd_detect_mult: NotRequired[int]  # BFD detection multiplier.
    bfd_required_min_rx: NotRequired[int]  # BFD required minimal receive interval.
    l2forward: NotRequired[Literal["enable", "disable"]]  # Enable/disable l2 forwarding.
    icmp_send_redirect: NotRequired[Literal["enable", "disable"]]  # Enable/disable sending of ICMP redirects.
    icmp_accept_redirect: NotRequired[Literal["enable", "disable"]]  # Enable/disable ICMP accept redirect.
    reachable_time: NotRequired[int]  # IPv4 reachable time in milliseconds (30000 - 3600000, defaul
    vlanforward: NotRequired[Literal["enable", "disable"]]  # Enable/disable traffic forwarding between VLANs on this inte
    stpforward: NotRequired[Literal["enable", "disable"]]  # Enable/disable STP forwarding.
    stpforward_mode: NotRequired[Literal["rpl-all-ext-id", "rpl-bridge-ext-id", "rpl-nothing"]]  # Configure STP forwarding mode.
    ips_sniffer_mode: NotRequired[Literal["enable", "disable"]]  # Enable/disable the use of this interface as a one-armed snif
    ident_accept: NotRequired[Literal["enable", "disable"]]  # Enable/disable authentication for this interface.
    ipmac: NotRequired[Literal["enable", "disable"]]  # Enable/disable IP/MAC binding.
    subst: NotRequired[Literal["enable", "disable"]]  # Enable to always send packets from this interface to a desti
    macaddr: NotRequired[str]  # Change the interface's MAC address.
    virtual_mac: NotRequired[str]  # Change the interface's virtual MAC address.
    substitute_dst_mac: NotRequired[str]  # Destination MAC address that all packets are sent to from th
    speed: NotRequired[Literal["auto", "10full", "10half", "100full", "100half", "100auto", "1000full", "1000auto"]]  # Interface speed. The default setting and the options availab
    status: NotRequired[Literal["up", "down"]]  # Bring the interface up or shut the interface down.
    netbios_forward: NotRequired[Literal["disable", "enable"]]  # Enable/disable NETBIOS forwarding.
    wins_ip: NotRequired[str]  # WINS server IP.
    type: NotRequired[Literal["physical", "vlan", "aggregate", "redundant", "tunnel", "vdom-link", "loopback", "switch", "vap-switch", "wl-mesh", "fext-wan", "vxlan", "geneve", "switch-vlan", "emac-vlan", "lan-extension"]]  # Interface type.
    dedicated_to: NotRequired[Literal["none", "management"]]  # Configure interface for single purpose.
    trust_ip_1: NotRequired[str]  # Trusted host for dedicated management traffic (0.0.0.0/24 fo
    trust_ip_2: NotRequired[str]  # Trusted host for dedicated management traffic (0.0.0.0/24 fo
    trust_ip_3: NotRequired[str]  # Trusted host for dedicated management traffic (0.0.0.0/24 fo
    trust_ip6_1: NotRequired[str]  # Trusted IPv6 host for dedicated management traffic (::/0 for
    trust_ip6_2: NotRequired[str]  # Trusted IPv6 host for dedicated management traffic (::/0 for
    trust_ip6_3: NotRequired[str]  # Trusted IPv6 host for dedicated management traffic (::/0 for
    ring_rx: NotRequired[int]  # RX ring size.
    ring_tx: NotRequired[int]  # TX ring size.
    wccp: NotRequired[Literal["enable", "disable"]]  # Enable/disable WCCP on this interface. Used for encapsulated
    netflow_sampler: NotRequired[Literal["disable", "tx", "rx", "both"]]  # Enable/disable NetFlow on this interface and set the data th
    netflow_sample_rate: NotRequired[int]  # NetFlow sample rate.  Sample one packet every configured num
    netflow_sampler_id: NotRequired[int]  # Netflow sampler ID.
    sflow_sampler: NotRequired[Literal["enable", "disable"]]  # Enable/disable sFlow on this interface.
    drop_fragment: NotRequired[Literal["enable", "disable"]]  # Enable/disable drop fragment packets.
    src_check: NotRequired[Literal["enable", "disable"]]  # Enable/disable source IP check.
    sample_rate: NotRequired[int]  # sFlow sample rate (10 - 99999).
    polling_interval: NotRequired[int]  # sFlow polling interval in seconds (1 - 255).
    sample_direction: NotRequired[Literal["tx", "rx", "both"]]  # Data that NetFlow collects (rx, tx, or both).
    explicit_web_proxy: NotRequired[Literal["enable", "disable"]]  # Enable/disable the explicit web proxy on this interface.
    explicit_ftp_proxy: NotRequired[Literal["enable", "disable"]]  # Enable/disable the explicit FTP proxy on this interface.
    proxy_captive_portal: NotRequired[Literal["enable", "disable"]]  # Enable/disable proxy captive portal on this interface.
    tcp_mss: NotRequired[int]  # TCP maximum segment size. 0 means do not change segment size
    inbandwidth: NotRequired[int]  # Bandwidth limit for incoming traffic (0 - 80000000 kbps), 0 
    outbandwidth: NotRequired[int]  # Bandwidth limit for outgoing traffic (0 - 80000000 kbps).
    egress_shaping_profile: NotRequired[str]  # Outgoing traffic shaping profile.
    ingress_shaping_profile: NotRequired[str]  # Incoming traffic shaping profile.
    spillover_threshold: NotRequired[int]  # Egress Spillover threshold (0 - 16776000 kbps), 0 means unli
    ingress_spillover_threshold: NotRequired[int]  # Ingress Spillover threshold (0 - 16776000 kbps), 0 means unl
    weight: NotRequired[int]  # Default weight for static routes (if route has no weight con
    interface: str  # Interface name.
    external: NotRequired[Literal["enable", "disable"]]  # Enable/disable identifying the interface as an external inte
    mtu_override: NotRequired[Literal["enable", "disable"]]  # Enable to set a custom MTU for this interface.
    mtu: NotRequired[int]  # MTU value for this interface.
    vlan_protocol: NotRequired[Literal["8021q", "8021ad"]]  # Ethernet protocol of VLAN.
    vlanid: NotRequired[int]  # VLAN ID (1 - 4094).
    forward_domain: NotRequired[int]  # Transparent mode forward domain.
    remote_ip: NotRequired[str]  # Remote IP address of tunnel.
    member: NotRequired[list[dict[str, Any]]]  # Physical interfaces that belong to the aggregate or redundan
    lacp_mode: NotRequired[Literal["static", "passive", "active"]]  # LACP mode.
    lacp_ha_secondary: NotRequired[Literal["enable", "disable"]]  # LACP HA secondary member.
    system_id_type: NotRequired[Literal["auto", "user"]]  # Method in which system ID is generated.
    system_id: str  # Define a system ID for the aggregate interface.
    lacp_speed: NotRequired[Literal["slow", "fast"]]  # How often the interface sends LACP messages.
    min_links: NotRequired[int]  # Minimum number of aggregated ports that must be up.
    min_links_down: NotRequired[Literal["operational", "administrative"]]  # Action to take when less than the configured minimum number 
    algorithm: NotRequired[Literal["L2", "L3", "L4", "NPU-GRE", "Source-MAC"]]  # Frame distribution algorithm.
    link_up_delay: NotRequired[int]  # Number of milliseconds to wait before considering a link is 
    aggregate_type: NotRequired[Literal["physical", "vxlan"]]  # Type of aggregation.
    priority_override: NotRequired[Literal["enable", "disable"]]  # Enable/disable fail back to higher priority port once recove
    aggregate: NotRequired[str]  # Aggregate interface.
    redundant_interface: NotRequired[str]  # Redundant interface.
    devindex: NotRequired[int]  # Device Index.
    vindex: NotRequired[int]  # Switch control interface VLAN ID.
    switch: NotRequired[str]  # Contained in switch.
    description: NotRequired[str]  # Description.
    alias: NotRequired[str]  # Alias will be displayed with the interface name to make it e
    security_mode: NotRequired[Literal["none", "captive-portal", "802.1X"]]  # Turn on captive portal authentication for this interface.
    security_mac_auth_bypass: NotRequired[Literal["mac-auth-only", "enable", "disable"]]  # Enable/disable MAC authentication bypass.
    security_ip_auth_bypass: NotRequired[Literal["enable", "disable"]]  # Enable/disable IP authentication bypass.
    security_external_web: NotRequired[str]  # URL of external authentication web server.
    security_external_logout: NotRequired[str]  # URL of external authentication logout server.
    replacemsg_override_group: NotRequired[str]  # Replacement message override group.
    security_redirect_url: NotRequired[str]  # URL redirection after disclaimer/authentication.
    auth_cert: NotRequired[str]  # HTTPS server certificate.
    auth_portal_addr: NotRequired[str]  # Address of captive portal.
    security_exempt_list: NotRequired[str]  # Name of security-exempt-list.
    security_groups: NotRequired[list[dict[str, Any]]]  # User groups that can authenticate with the captive portal.
    ike_saml_server: NotRequired[str]  # Configure IKE authentication SAML server.
    device_identification: NotRequired[Literal["enable", "disable"]]  # Enable/disable passively gathering of device identity inform
    exclude_signatures: NotRequired[Literal["iot", "ot"]]  # Exclude IOT or OT application signatures.
    device_user_identification: NotRequired[Literal["enable", "disable"]]  # Enable/disable passive gathering of user identity informatio
    lldp_reception: NotRequired[Literal["enable", "disable", "vdom"]]  # Enable/disable Link Layer Discovery Protocol (LLDP) receptio
    lldp_transmission: NotRequired[Literal["enable", "disable", "vdom"]]  # Enable/disable Link Layer Discovery Protocol (LLDP) transmis
    lldp_network_policy: NotRequired[str]  # LLDP-MED network policy profile.
    estimated_upstream_bandwidth: NotRequired[int]  # Estimated maximum upstream bandwidth (kbps). Used to estimat
    estimated_downstream_bandwidth: NotRequired[int]  # Estimated maximum downstream bandwidth (kbps). Used to estim
    measured_upstream_bandwidth: NotRequired[int]  # Measured upstream bandwidth (kbps).
    measured_downstream_bandwidth: NotRequired[int]  # Measured downstream bandwidth (kbps).
    bandwidth_measure_time: NotRequired[int]  # Bandwidth measure time.
    monitor_bandwidth: NotRequired[Literal["enable", "disable"]]  # Enable monitoring bandwidth on this interface.
    vrrp_virtual_mac: NotRequired[Literal["enable", "disable"]]  # Enable/disable use of virtual MAC for VRRP.
    vrrp: NotRequired[list[dict[str, Any]]]  # VRRP configuration.
    role: NotRequired[Literal["lan", "wan", "dmz", "undefined"]]  # Interface role.
    snmp_index: NotRequired[int]  # Permanent SNMP Index of the interface.
    secondary_IP: NotRequired[Literal["enable", "disable"]]  # Enable/disable adding a secondary IP to this interface.
    secondaryip: NotRequired[list[dict[str, Any]]]  # Second IP address of interface.
    preserve_session_route: NotRequired[Literal["enable", "disable"]]  # Enable/disable preservation of session route when dirty.
    auto_auth_extension_device: NotRequired[Literal["enable", "disable"]]  # Enable/disable automatic authorization of dedicated Fortinet
    ap_discover: NotRequired[Literal["enable", "disable"]]  # Enable/disable automatic registration of unknown FortiAP dev
    fortilink_neighbor_detect: NotRequired[Literal["lldp", "fortilink"]]  # Protocol for FortiGate neighbor discovery.
    ip_managed_by_fortiipam: NotRequired[Literal["inherit-global", "enable", "disable"]]  # Enable/disable automatic IP address assignment of this inter
    managed_subnetwork_size: NotRequired[Literal["4", "8", "16", "32", "64", "128", "256", "512", "1024", "2048", "4096", "8192", "16384", "32768", "65536", "131072", "262144", "524288", "1048576", "2097152", "4194304", "8388608", "16777216"]]  # Number of IP addresses to be allocated by FortiIPAM and used
    fortilink_split_interface: NotRequired[Literal["enable", "disable"]]  # Enable/disable FortiLink split interface to connect member l
    internal: NotRequired[int]  # Implicitly created.
    fortilink_backup_link: NotRequired[int]  # FortiLink split interface backup link.
    switch_controller_access_vlan: NotRequired[Literal["enable", "disable"]]  # Block FortiSwitch port-to-port traffic.
    switch_controller_traffic_policy: NotRequired[str]  # Switch controller traffic policy for the VLAN.
    switch_controller_rspan_mode: NotRequired[Literal["disable", "enable"]]  # Stop Layer2 MAC learning and interception of BPDUs and other
    switch_controller_netflow_collect: NotRequired[Literal["disable", "enable"]]  # NetFlow collection and processing.
    switch_controller_mgmt_vlan: NotRequired[int]  # VLAN to use for FortiLink management purposes.
    switch_controller_igmp_snooping: NotRequired[Literal["enable", "disable"]]  # Switch controller IGMP snooping.
    switch_controller_igmp_snooping_proxy: NotRequired[Literal["enable", "disable"]]  # Switch controller IGMP snooping proxy.
    switch_controller_igmp_snooping_fast_leave: NotRequired[Literal["enable", "disable"]]  # Switch controller IGMP snooping fast-leave.
    switch_controller_dhcp_snooping: NotRequired[Literal["enable", "disable"]]  # Switch controller DHCP snooping.
    switch_controller_dhcp_snooping_verify_mac: NotRequired[Literal["enable", "disable"]]  # Switch controller DHCP snooping verify MAC.
    switch_controller_dhcp_snooping_option82: NotRequired[Literal["enable", "disable"]]  # Switch controller DHCP snooping option82.
    dhcp_snooping_server_list: NotRequired[list[dict[str, Any]]]  # Configure DHCP server access list.
    switch_controller_arp_inspection: NotRequired[Literal["enable", "disable", "monitor"]]  # Enable/disable/Monitor FortiSwitch ARP inspection.
    switch_controller_learning_limit: NotRequired[int]  # Limit the number of dynamic MAC addresses on this VLAN (1 - 
    switch_controller_nac: NotRequired[str]  # Integrated FortiLink settings for managed FortiSwitch.
    switch_controller_dynamic: NotRequired[str]  # Integrated FortiLink settings for managed FortiSwitch.
    switch_controller_feature: NotRequired[Literal["none", "default-vlan", "quarantine", "rspan", "voice", "video", "nac", "nac-segment"]]  # Interface's purpose when assigning traffic (read only).
    switch_controller_iot_scanning: NotRequired[Literal["enable", "disable"]]  # Enable/disable managed FortiSwitch IoT scanning.
    switch_controller_offload: NotRequired[Literal["enable", "disable"]]  # Enable/disable managed FortiSwitch routing offload.
    switch_controller_offload_ip: NotRequired[str]  # IP for routing offload on FortiSwitch.
    switch_controller_offload_gw: NotRequired[Literal["enable", "disable"]]  # Enable/disable managed FortiSwitch routing offload gateway.
    swc_vlan: NotRequired[int]  # Creation status for switch-controller VLANs.
    swc_first_create: NotRequired[int]  # Initial create for switch-controller VLANs.
    color: NotRequired[int]  # Color of icon on the GUI.
    tagging: NotRequired[list[dict[str, Any]]]  # Config object tagging.
    eap_supplicant: NotRequired[Literal["enable", "disable"]]  # Enable/disable EAP-Supplicant.
    eap_method: NotRequired[Literal["tls", "peap"]]  # EAP method.
    eap_identity: NotRequired[str]  # EAP identity.
    eap_password: NotRequired[str]  # EAP password.
    eap_ca_cert: NotRequired[str]  # EAP CA certificate name.
    eap_user_cert: NotRequired[str]  # EAP user certificate name.
    default_purdue_level: NotRequired[Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]]  # default purdue level of device detected on this interface.
    ipv6: NotRequired[str]  # IPv6 of interface.
    physical: NotRequired[str]  # Print physical interface information.


class Interface:
    """
    Configure interfaces.
    
    Path: system/interface
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: InterfacePayload | None = ...,
        name: str | None = ...,
        vrf: int | None = ...,
        cli_conn_status: int | None = ...,
        fortilink: Literal["enable", "disable"] | None = ...,
        switch_controller_source_ip: Literal["outbound", "fixed"] | None = ...,
        mode: Literal["static", "dhcp", "pppoe"] | None = ...,
        client_options: list[dict[str, Any]] | None = ...,
        distance: int | None = ...,
        priority: int | None = ...,
        dhcp_relay_interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        dhcp_relay_interface: str | None = ...,
        dhcp_relay_vrf_select: int | None = ...,
        dhcp_broadcast_flag: Literal["disable", "enable"] | None = ...,
        dhcp_relay_service: Literal["disable", "enable"] | None = ...,
        dhcp_relay_ip: str | None = ...,
        dhcp_relay_source_ip: str | None = ...,
        dhcp_relay_circuit_id: str | None = ...,
        dhcp_relay_link_selection: str | None = ...,
        dhcp_relay_request_all_server: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: InterfacePayload | None = ...,
        name: str | None = ...,
        vrf: int | None = ...,
        cli_conn_status: int | None = ...,
        fortilink: Literal["enable", "disable"] | None = ...,
        switch_controller_source_ip: Literal["outbound", "fixed"] | None = ...,
        mode: Literal["static", "dhcp", "pppoe"] | None = ...,
        client_options: list[dict[str, Any]] | None = ...,
        distance: int | None = ...,
        priority: int | None = ...,
        dhcp_relay_interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        dhcp_relay_interface: str | None = ...,
        dhcp_relay_vrf_select: int | None = ...,
        dhcp_broadcast_flag: Literal["disable", "enable"] | None = ...,
        dhcp_relay_service: Literal["disable", "enable"] | None = ...,
        dhcp_relay_ip: str | None = ...,
        dhcp_relay_source_ip: str | None = ...,
        dhcp_relay_circuit_id: str | None = ...,
        dhcp_relay_link_selection: str | None = ...,
        dhcp_relay_request_all_server: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: InterfacePayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Interface",
    "InterfacePayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class FabricVpnPayload(TypedDict, total=False):
    """
    Type hints for system/fabric_vpn payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: FabricVpnPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    status: Literal["enable", "disable"]  # Enable/disable Fabric VPN.
    sync_mode: Literal["enable", "disable"]  # Setting synchronised by fabric or manual.
    branch_name: NotRequired[str]  # Branch name.
    policy_rule: NotRequired[Literal["health-check", "manual", "auto"]]  # Policy creation rule.
    vpn_role: Literal["hub", "spoke"]  # Fabric VPN role.
    overlays: NotRequired[list[dict[str, Any]]]  # Local overlay interfaces table.
    advertised_subnets: NotRequired[list[dict[str, Any]]]  # Local advertised subnets.
    loopback_address_block: str  # IPv4 address and subnet mask for hub's loopback address, syn
    loopback_interface: NotRequired[str]  # Loopback interface.
    loopback_advertised_subnet: NotRequired[int]  # Loopback advertised subnet reference.
    psksecret: str  # Pre-shared secret for ADVPN.
    bgp_as: str  # BGP Router AS number, asplain/asdot/asdot+ format.
    sdwan_zone: NotRequired[str]  # Reference to created SD-WAN zone.
    health_checks: NotRequired[str]  # Underlying health checks.


class FabricVpn:
    """
    Setup for self orchestrated fabric auto discovery VPN.
    
    Path: system/fabric_vpn
    Category: cmdb
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: FabricVpnPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        sync_mode: Literal["enable", "disable"] | None = ...,
        branch_name: str | None = ...,
        policy_rule: Literal["health-check", "manual", "auto"] | None = ...,
        vpn_role: Literal["hub", "spoke"] | None = ...,
        overlays: list[dict[str, Any]] | None = ...,
        advertised_subnets: list[dict[str, Any]] | None = ...,
        loopback_address_block: str | None = ...,
        loopback_interface: str | None = ...,
        loopback_advertised_subnet: int | None = ...,
        psksecret: str | None = ...,
        bgp_as: str | None = ...,
        sdwan_zone: str | None = ...,
        health_checks: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: FabricVpnPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        sync_mode: Literal["enable", "disable"] | None = ...,
        branch_name: str | None = ...,
        policy_rule: Literal["health-check", "manual", "auto"] | None = ...,
        vpn_role: Literal["hub", "spoke"] | None = ...,
        overlays: list[dict[str, Any]] | None = ...,
        advertised_subnets: list[dict[str, Any]] | None = ...,
        loopback_address_block: str | None = ...,
        loopback_interface: str | None = ...,
        loopback_advertised_subnet: int | None = ...,
        psksecret: str | None = ...,
        bgp_as: str | None = ...,
        sdwan_zone: str | None = ...,
        health_checks: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: FabricVpnPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "FabricVpn",
    "FabricVpnPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class NacPolicyPayload(TypedDict, total=False):
    """
    Type hints for user/nac_policy payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: NacPolicyPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: NotRequired[str]  # NAC policy name.
    description: NotRequired[str]  # Description for the NAC policy matching pattern.
    category: NotRequired[Literal["device", "firewall-user", "ems-tag", "fortivoice-tag", "vulnerability"]]  # Category of NAC policy.
    status: NotRequired[Literal["enable", "disable"]]  # Enable/disable NAC policy.
    match_type: NotRequired[Literal["dynamic", "override"]]  # Match and retain the devices based on the type.
    match_period: NotRequired[int]  # Number of days the matched devices will be retained (0 - alw
    match_remove: NotRequired[Literal["default", "link-down"]]  # Options to remove the matched override devices.
    mac: NotRequired[str]  # NAC policy matching MAC address.
    hw_vendor: NotRequired[str]  # NAC policy matching hardware vendor.
    type: NotRequired[str]  # NAC policy matching type.
    family: NotRequired[str]  # NAC policy matching family.
    os: NotRequired[str]  # NAC policy matching operating system.
    hw_version: NotRequired[str]  # NAC policy matching hardware version.
    sw_version: NotRequired[str]  # NAC policy matching software version.
    host: NotRequired[str]  # NAC policy matching host.
    user: NotRequired[str]  # NAC policy matching user.
    src: NotRequired[str]  # NAC policy matching source.
    user_group: NotRequired[str]  # NAC policy matching user group.
    ems_tag: NotRequired[str]  # NAC policy matching EMS tag.
    fortivoice_tag: NotRequired[str]  # NAC policy matching FortiVoice tag.
    severity: NotRequired[list[dict[str, Any]]]  # NAC policy matching devices vulnerability severity lists.
    switch_fortilink: NotRequired[str]  # FortiLink interface for which this NAC policy belongs to.
    switch_group: NotRequired[list[dict[str, Any]]]  # List of managed FortiSwitch groups on which NAC policy can b
    switch_mac_policy: NotRequired[str]  # Switch MAC policy action to be applied on the matched NAC po
    firewall_address: NotRequired[str]  # Dynamic firewall address to associate MAC which match this p
    ssid_policy: NotRequired[str]  # SSID policy to be applied on the matched NAC policy.


class NacPolicy:
    """
    Configure NAC policy matching pattern to identify matching NAC devices.
    
    Path: user/nac_policy
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: NacPolicyPayload | None = ...,
        name: str | None = ...,
        description: str | None = ...,
        category: Literal["device", "firewall-user", "ems-tag", "fortivoice-tag", "vulnerability"] | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        match_type: Literal["dynamic", "override"] | None = ...,
        match_period: int | None = ...,
        match_remove: Literal["default", "link-down"] | None = ...,
        mac: str | None = ...,
        hw_vendor: str | None = ...,
        type: str | None = ...,
        family: str | None = ...,
        os: str | None = ...,
        hw_version: str | None = ...,
        sw_version: str | None = ...,
        host: str | None = ...,
        user: str | None = ...,
        src: str | None = ...,
        user_group: str | None = ...,
        ems_tag: str | None = ...,
        fortivoice_tag: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: NacPolicyPayload | None = ...,
        name: str | None = ...,
        description: str | None = ...,
        category: Literal["device", "firewall-user", "ems-tag", "fortivoice-tag", "vulnerability"] | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        match_type: Literal["dynamic", "override"] | None = ...,
        match_period: int | None = ...,
        match_remove: Literal["default", "link-down"] | None = ...,
        mac: str | None = ...,
        hw_vendor: str | None = ...,
        type: str | None = ...,
        family: str | None = ...,
        os: str | None = ...,
        hw_version: str | None = ...,
        sw_version: str | None = ...,
        host: str | None = ...,
        user: str | None = ...,
        src: str | None = ...,
        user_group: str | None = ...,
        ems_tag: str | None = ...,
        fortivoice_tag: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: NacPolicyPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "NacPolicy",
    "NacPolicyPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class FortiguardPayload(TypedDict, total=False):
    """
    Type hints for webfilter/fortiguard payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: FortiguardPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    cache_mode: NotRequired[Literal["ttl", "db-ver"]]  # Cache entry expiration mode.
    cache_prefix_match: NotRequired[Literal["enable", "disable"]]  # Enable/disable prefix matching in the cache.
    cache_mem_permille: NotRequired[int]  # Maximum permille of available memory allocated to caching (1
    ovrd_auth_port_http: NotRequired[int]  # Port to use for FortiGuard Web Filter HTTP override authenti
    ovrd_auth_port_https: NotRequired[int]  # Port to use for FortiGuard Web Filter HTTPS override authent
    ovrd_auth_port_https_flow: NotRequired[int]  # Port to use for FortiGuard Web Filter HTTPS override authent
    ovrd_auth_port_warning: NotRequired[int]  # Port to use for FortiGuard Web Filter Warning override authe
    ovrd_auth_https: NotRequired[Literal["enable", "disable"]]  # Enable/disable use of HTTPS for override authentication.
    warn_auth_https: NotRequired[Literal["enable", "disable"]]  # Enable/disable use of HTTPS for warning and authentication.
    close_ports: NotRequired[Literal["enable", "disable"]]  # Close ports used for HTTP/HTTPS override authentication and 
    request_packet_size_limit: NotRequired[int]  # Limit size of URL request packets sent to FortiGuard server 
    embed_image: NotRequired[Literal["enable", "disable"]]  # Enable/disable embedding images into replacement messages (d


class Fortiguard:
    """
    Configure FortiGuard Web Filter service.
    
    Path: webfilter/fortiguard
    Category: cmdb
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: FortiguardPayload | None = ...,
        cache_mode: Literal["ttl", "db-ver"] | None = ...,
        cache_prefix_match: Literal["enable", "disable"] | None = ...,
        cache_mem_permille: int | None = ...,
        ovrd_auth_port_http: int | None = ...,
        ovrd_auth_port_https: int | None = ...,
        ovrd_auth_port_https_flow: int | None = ...,
        ovrd_auth_port_warning: int | None = ...,
        ovrd_auth_https: Literal["enable", "disable"] | None = ...,
        warn_auth_https: Literal["enable", "disable"] | None = ...,
        close_ports: Literal["enable", "disable"] | None = ...,
        request_packet_size_limit: int | None = ...,
        embed_image: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: FortiguardPayload | None = ...,
        cache_mode: Literal["ttl", "db-ver"] | None = ...,
        cache_prefix_match: Literal["enable", "disable"] | None = ...,
        cache_mem_permille: int | None = ...,
        ovrd_auth_port_http: int | None = ...,
        ovrd_auth_port_https: int | None = ...,
        ovrd_auth_port_https_flow: int | None = ...,
        ovrd_auth_port_warning: int | None = ...,
        ovrd_auth_https: Literal["enable", "disable"] | None = ...,
        warn_auth_https: Literal["enable", "disable"] | None = ...,
        close_ports: Literal["enable", "disable"] | None = ...,
        request_packet_size_limit: int | None = ...,
        embed_image: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: FortiguardPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Fortiguard",
    "FortiguardPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class SpeedTestSchedulePayload(TypedDict, total=False):
    """
    Type hints for system/speed_test_schedule payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: SpeedTestSchedulePayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    interface: NotRequired[str]  # Interface name.
    status: NotRequired[Literal["disable", "enable"]]  # Enable/disable scheduled speed test.
    diffserv: NotRequired[str]  # DSCP used for speed test.
    server_name: NotRequired[str]  # Speed test server name.
    mode: NotRequired[Literal["UDP", "TCP", "Auto"]]  # Protocol Auto(default), TCP or UDP used for speed test.
    schedules: list[dict[str, Any]]  # Schedules for the interface.
    dynamic_server: NotRequired[Literal["disable", "enable"]]  # Enable/disable dynamic server option.
    ctrl_port: NotRequired[int]  # Port of the controller to get access token.
    server_port: NotRequired[int]  # Port of the server to run speed test.
    update_shaper: NotRequired[Literal["disable", "local", "remote", "both"]]  # Set egress shaper based on the test result.
    update_inbandwidth: NotRequired[Literal["disable", "enable"]]  # Enable/disable bypassing interface's inbound bandwidth setti
    update_outbandwidth: NotRequired[Literal["disable", "enable"]]  # Enable/disable bypassing interface's outbound bandwidth sett
    update_inbandwidth_maximum: NotRequired[int]  # Maximum downloading bandwidth (kbps) to be used in a speed t
    update_inbandwidth_minimum: NotRequired[int]  # Minimum downloading bandwidth (kbps) to be considered effect
    update_outbandwidth_maximum: NotRequired[int]  # Maximum uploading bandwidth (kbps) to be used in a speed tes
    update_outbandwidth_minimum: NotRequired[int]  # Minimum uploading bandwidth (kbps) to be considered effectiv


class SpeedTestSchedule:
    """
    Speed test schedule for each interface.
    
    Path: system/speed_test_schedule
    Category: cmdb
    Primary Key: interface
    """
    
    def get(
        self,
        interface: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: SpeedTestSchedulePayload | None = ...,
        interface: str | None = ...,
        status: Literal["disable", "enable"] | None = ...,
        diffserv: str | None = ...,
        server_name: str | None = ...,
        mode: Literal["UDP", "TCP", "Auto"] | None = ...,
        schedules: list[dict[str, Any]] | None = ...,
        dynamic_server: Literal["disable", "enable"] | None = ...,
        ctrl_port: int | None = ...,
        server_port: int | None = ...,
        update_shaper: Literal["disable", "local", "remote", "both"] | None = ...,
        update_inbandwidth: Literal["disable", "enable"] | None = ...,
        update_outbandwidth: Literal["disable", "enable"] | None = ...,
        update_inbandwidth_maximum: int | None = ...,
        update_inbandwidth_minimum: int | None = ...,
        update_outbandwidth_maximum: int | None = ...,
        update_outbandwidth_minimum: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: SpeedTestSchedulePayload | None = ...,
        interface: str | None = ...,
        status: Literal["disable", "enable"] | None = ...,
        diffserv: str | None = ...,
        server_name: str | None = ...,
        mode: Literal["UDP", "TCP", "Auto"] | None = ...,
        schedules: list[dict[str, Any]] | None = ...,
        dynamic_server: Literal["disable", "enable"] | None = ...,
        ctrl_port: int | None = ...,
        server_port: int | None = ...,
        update_shaper: Literal["disable", "local", "remote", "both"] | None = ...,
        update_inbandwidth: Literal["disable", "enable"] | None = ...,
        update_outbandwidth: Literal["disable", "enable"] | None = ...,
        update_inbandwidth_maximum: int | None = ...,
        update_inbandwidth_minimum: int | None = ...,
        update_outbandwidth_maximum: int | None = ...,
        update_outbandwidth_minimum: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        interface: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        interface: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: SpeedTestSchedulePayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "SpeedTestSchedule",
    "SpeedTestSchedulePayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class GreTunnelPayload(TypedDict, total=False):
    """
    Type hints for system/gre_tunnel payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: GreTunnelPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: NotRequired[str]  # Tunnel name.
    interface: NotRequired[str]  # Interface name.
    ip_version: NotRequired[Literal["4", "6"]]  # IP version to use for VPN interface.
    remote_gw6: str  # IPv6 address of the remote gateway.
    local_gw6: str  # IPv6 address of the local gateway.
    remote_gw: str  # IP address of the remote gateway.
    local_gw: str  # IP address of the local gateway.
    use_sdwan: NotRequired[Literal["disable", "enable"]]  # Enable/disable use of SD-WAN to reach remote gateway.
    sequence_number_transmission: NotRequired[Literal["disable", "enable"]]  # Enable/disable including of sequence numbers in transmitted 
    sequence_number_reception: NotRequired[Literal["disable", "enable"]]  # Enable/disable validating sequence numbers in received GRE p
    checksum_transmission: NotRequired[Literal["disable", "enable"]]  # Enable/disable including checksums in transmitted GRE packet
    checksum_reception: NotRequired[Literal["disable", "enable"]]  # Enable/disable validating checksums in received GRE packets.
    key_outbound: NotRequired[int]  # Include this key in transmitted GRE packets (0 - 4294967295)
    key_inbound: NotRequired[int]  # Require received GRE packets contain this key (0 - 429496729
    dscp_copying: NotRequired[Literal["disable", "enable"]]  # Enable/disable DSCP copying.
    diffservcode: NotRequired[str]  # DiffServ setting to be applied to GRE tunnel outer IP header
    keepalive_interval: NotRequired[int]  # Keepalive message interval (0 - 32767, 0 = disabled).
    keepalive_failtimes: NotRequired[int]  # Number of consecutive unreturned keepalive messages before a


class GreTunnel:
    """
    Configure GRE tunnel.
    
    Path: system/gre_tunnel
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: GreTunnelPayload | None = ...,
        name: str | None = ...,
        interface: str | None = ...,
        ip_version: Literal["4", "6"] | None = ...,
        remote_gw6: str | None = ...,
        local_gw6: str | None = ...,
        remote_gw: str | None = ...,
        local_gw: str | None = ...,
        use_sdwan: Literal["disable", "enable"] | None = ...,
        sequence_number_transmission: Literal["disable", "enable"] | None = ...,
        sequence_number_reception: Literal["disable", "enable"] | None = ...,
        checksum_transmission: Literal["disable", "enable"] | None = ...,
        checksum_reception: Literal["disable", "enable"] | None = ...,
        key_outbound: int | None = ...,
        key_inbound: int | None = ...,
        dscp_copying: Literal["disable", "enable"] | None = ...,
        diffservcode: str | None = ...,
        keepalive_interval: int | None = ...,
        keepalive_failtimes: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: GreTunnelPayload | None = ...,
        name: str | None = ...,
        interface: str | None = ...,
        ip_version: Literal["4", "6"] | None = ...,
        remote_gw6: str | None = ...,
        local_gw6: str | None = ...,
        remote_gw: str | None = ...,
        local_gw: str | None = ...,
        use_sdwan: Literal["disable", "enable"] | None = ...,
        sequence_number_transmission: Literal["disable", "enable"] | None = ...,
        sequence_number_reception: Literal["disable", "enable"] | None = ...,
        checksum_transmission: Literal["disable", "enable"] | None = ...,
        checksum_reception: Literal["disable", "enable"] | None = ...,
        key_outbound: int | None = ...,
        key_inbound: int | None = ...,
        dscp_copying: Literal["disable", "enable"] | None = ...,
        diffservcode: str | None = ...,
        keepalive_interval: int | None = ...,
        keepalive_failtimes: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: GreTunnelPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "GreTunnel",
    "GreTunnelPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class RemoteLogPayload(TypedDict, total=False):
    """
    Type hints for switch_controller/remote_log payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: RemoteLogPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: str  # Remote log name.
    status: NotRequired[Literal["enable", "disable"]]  # Enable/disable logging by FortiSwitch device to a remote sys
    server: str  # IPv4 address of the remote syslog server.
    port: NotRequired[int]  # Remote syslog server listening port.
    severity: NotRequired[Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]]  # Severity of logs to be transferred to remote log server.
    csv: NotRequired[Literal["enable", "disable"]]  # Enable/disable comma-separated value (CSV) strings.
    facility: NotRequired[Literal["kernel", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "ntp", "audit", "alert", "clock", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"]]  # Facility to log to remote syslog server.


class RemoteLog:
    """
    Configure logging by FortiSwitch device to a remote syslog server.
    
    Path: switch_controller/remote_log
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: RemoteLogPayload | None = ...,
        name: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        server: str | None = ...,
        port: int | None = ...,
        severity: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        csv: Literal["enable", "disable"] | None = ...,
        facility: Literal["kernel", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "ntp", "audit", "alert", "clock", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: RemoteLogPayload | None = ...,
        name: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        server: str | None = ...,
        port: int | None = ...,
        severity: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        csv: Literal["enable", "disable"] | None = ...,
        facility: Literal["kernel", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "ntp", "audit", "alert", "clock", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: RemoteLogPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "RemoteLog",
    "RemoteLogPayload",
]


"""
Validation helpers for antivirus/profile endpoint.

Each endpoint has its own validation file to keep validation logic
separate and maintainable. Use central cmdb._helpers tools for common tasks.

Auto-generated from OpenAPI specification
Customize as needed for endpoint-specific business logic.
"""

from typing import Any, TypedDict, NotRequired, Literal

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:
#
# 1. FALSE POSITIVES: Some fields marked "required" have default values,
#    meaning they're optional (filtered out by generator)
#
# 2. CONDITIONAL REQUIREMENTS: Many endpoints require EITHER field A OR field B:
#    - firewall.policy: requires (srcaddr + dstaddr) OR (srcaddr6 + dstaddr6)
#    - These conditional requirements cannot be expressed in a simple list
#
# 3. SPECIALIZED FEATURES: Fields for WAN optimization, VPN, NAT64, etc.
#    are marked "required" but only apply when using those features
#
# The REQUIRED_FIELDS list below is INFORMATIONAL ONLY and shows fields that:
# - Are marked required in the schema
# - Don't have non-empty default values
# - Aren't specialized feature fields
#
# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "name",  # Profile name.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "replacemsg-group": "",
    "feature-set": "flow",
    "fortisandbox-mode": "analytics-everything",
    "fortisandbox-max-upload": 10,
    "analytics-ignore-filetype": 0,
    "analytics-accept-filetype": 0,
    "analytics-db": "disable",
    "mobile-malware-db": "enable",
    "outbreak-prevention-archive-scan": "enable",
    "external-blocklist-enable-all": "disable",
    "ems-threat-feed": "disable",
    "fortindr-error-action": "log-only",
    "fortindr-timeout-action": "log-only",
    "fortisandbox-error-action": "log-only",
    "fortisandbox-timeout-action": "log-only",
    "av-virus-log": "enable",
    "extended-log": "disable",
    "scan-mode": "default",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Profile name.
    "comment": "var-string",  # Comment.
    "replacemsg-group": "string",  # Replacement message group customized for this profile.
    "feature-set": "option",  # Flow/proxy feature set.
    "fortisandbox-mode": "option",  # FortiSandbox scan modes.
    "fortisandbox-max-upload": "integer",  # Maximum size of files that can be uploaded to FortiSandbox i
    "analytics-ignore-filetype": "integer",  # Do not submit files matching this DLP file-pattern to FortiS
    "analytics-accept-filetype": "integer",  # Only submit files matching this DLP file-pattern to FortiSan
    "analytics-db": "option",  # Enable/disable using the FortiSandbox signature database to 
    "mobile-malware-db": "option",  # Enable/disable using the mobile malware signature database.
    "http": "string",  # Configure HTTP AntiVirus options.
    "ftp": "string",  # Configure FTP AntiVirus options.
    "imap": "string",  # Configure IMAP AntiVirus options.
    "pop3": "string",  # Configure POP3 AntiVirus options.
    "smtp": "string",  # Configure SMTP AntiVirus options.
    "mapi": "string",  # Configure MAPI AntiVirus options.
    "nntp": "string",  # Configure NNTP AntiVirus options.
    "cifs": "string",  # Configure CIFS AntiVirus options.
    "ssh": "string",  # Configure SFTP and SCP AntiVirus options.
    "nac-quar": "string",  # Configure AntiVirus quarantine settings.
    "content-disarm": "string",  # AV Content Disarm and Reconstruction settings.
    "outbreak-prevention-archive-scan": "option",  # Enable/disable outbreak-prevention archive scanning.
    "external-blocklist-enable-all": "option",  # Enable/disable all external blocklists.
    "external-blocklist": "string",  # One or more external malware block lists.
    "ems-threat-feed": "option",  # Enable/disable use of EMS threat feed when performing AntiVi
    "fortindr-error-action": "option",  # Action to take if FortiNDR encounters an error.
    "fortindr-timeout-action": "option",  # Action to take if FortiNDR encounters a scan timeout.
    "fortisandbox-error-action": "option",  # Action to take if FortiSandbox inline scan encounters an err
    "fortisandbox-timeout-action": "option",  # Action to take if FortiSandbox inline scan encounters a scan
    "av-virus-log": "option",  # Enable/disable AntiVirus logging.
    "extended-log": "option",  # Enable/disable extended logging for antivirus.
    "scan-mode": "option",  # Configure scan mode (default or legacy).
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Profile name.",
    "comment": "Comment.",
    "replacemsg-group": "Replacement message group customized for this profile.",
    "feature-set": "Flow/proxy feature set.",
    "fortisandbox-mode": "FortiSandbox scan modes.",
    "fortisandbox-max-upload": "Maximum size of files that can be uploaded to FortiSandbox in Mbytes.",
    "analytics-ignore-filetype": "Do not submit files matching this DLP file-pattern to FortiSandbox (post-transfer scan only).",
    "analytics-accept-filetype": "Only submit files matching this DLP file-pattern to FortiSandbox (post-transfer scan only).",
    "analytics-db": "Enable/disable using the FortiSandbox signature database to supplement the AV signature databases.",
    "mobile-malware-db": "Enable/disable using the mobile malware signature database.",
    "http": "Configure HTTP AntiVirus options.",
    "ftp": "Configure FTP AntiVirus options.",
    "imap": "Configure IMAP AntiVirus options.",
    "pop3": "Configure POP3 AntiVirus options.",
    "smtp": "Configure SMTP AntiVirus options.",
    "mapi": "Configure MAPI AntiVirus options.",
    "nntp": "Configure NNTP AntiVirus options.",
    "cifs": "Configure CIFS AntiVirus options.",
    "ssh": "Configure SFTP and SCP AntiVirus options.",
    "nac-quar": "Configure AntiVirus quarantine settings.",
    "content-disarm": "AV Content Disarm and Reconstruction settings.",
    "outbreak-prevention-archive-scan": "Enable/disable outbreak-prevention archive scanning.",
    "external-blocklist-enable-all": "Enable/disable all external blocklists.",
    "external-blocklist": "One or more external malware block lists.",
    "ems-threat-feed": "Enable/disable use of EMS threat feed when performing AntiVirus scan. Analyzes files including the content of archives.",
    "fortindr-error-action": "Action to take if FortiNDR encounters an error.",
    "fortindr-timeout-action": "Action to take if FortiNDR encounters a scan timeout.",
    "fortisandbox-error-action": "Action to take if FortiSandbox inline scan encounters an error.",
    "fortisandbox-timeout-action": "Action to take if FortiSandbox inline scan encounters a scan timeout.",
    "av-virus-log": "Enable/disable AntiVirus logging.",
    "extended-log": "Enable/disable extended logging for antivirus.",
    "scan-mode": "Configure scan mode (default or legacy).",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 47},
    "replacemsg-group": {"type": "string", "max_length": 35},
    "fortisandbox-max-upload": {"type": "integer", "min": 1, "max": 4095},
    "analytics-ignore-filetype": {"type": "integer", "min": 0, "max": 4294967295},
    "analytics-accept-filetype": {"type": "integer", "min": 0, "max": 4294967295},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "http": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "content-disarm": {
            "type": "option",
            "help": "Enable/disable Content Disarm and Reconstruction when performing AntiVirus scan.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
    },
    "ftp": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
    },
    "imap": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "executables": {
            "type": "option",
            "help": "Treat Windows executable files as viruses for the purpose of blocking or monitoring.",
            "default": "default",
            "options": ["default", "virus"],
        },
        "content-disarm": {
            "type": "option",
            "help": "Enable/disable Content Disarm and Reconstruction when performing AntiVirus scan.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
    },
    "pop3": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "executables": {
            "type": "option",
            "help": "Treat Windows executable files as viruses for the purpose of blocking or monitoring.",
            "default": "default",
            "options": ["default", "virus"],
        },
        "content-disarm": {
            "type": "option",
            "help": "Enable/disable Content Disarm and Reconstruction when performing AntiVirus scan.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
    },
    "smtp": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "executables": {
            "type": "option",
            "help": "Treat Windows executable files as viruses for the purpose of blocking or monitoring.",
            "default": "default",
            "options": ["default", "virus"],
        },
        "content-disarm": {
            "type": "option",
            "help": "Enable/disable Content Disarm and Reconstruction when performing AntiVirus scan.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
    },
    "mapi": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "executables": {
            "type": "option",
            "help": "Treat Windows executable files as viruses for the purpose of blocking or monitoring.",
            "default": "default",
            "options": ["default", "virus"],
        },
    },
    "nntp": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
    },
    "cifs": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
    },
    "ssh": {
        "av-scan": {
            "type": "option",
            "help": "Enable AntiVirus scan service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "outbreak-prevention": {
            "type": "option",
            "help": "Enable virus outbreak prevention service.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "external-blocklist": {
            "type": "option",
            "help": "Enable external-blocklist. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "malware-stream": {
            "type": "option",
            "help": "Enable 0-day malware-stream scanning. Analyzes files including the content of archives.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortindr": {
            "type": "option",
            "help": "Enable scanning of files by FortiNDR.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "fortisandbox": {
            "type": "option",
            "help": "Enable scanning of files by FortiSandbox.",
            "default": "disable",
            "options": ["disable", "block", "monitor"],
        },
        "quarantine": {
            "type": "option",
            "help": "Enable/disable quarantine for infected files.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
        "archive-block": {
            "type": "option",
            "help": "Select the archive types to block.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "archive-log": {
            "type": "option",
            "help": "Select the archive types to log.",
            "default": "",
            "options": ["encrypted", "corrupted", "partiallycorrupted", "multipart", "nested", "mailbomb", "timeout", "unhandled"],
        },
        "emulator": {
            "type": "option",
            "help": "Enable/disable the virus emulator.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
    },
    "nac-quar": {
        "infected": {
            "type": "option",
            "help": "Enable/Disable quarantining infected hosts to the banned user list.",
            "default": "none",
            "options": ["none", "quar-src-ip"],
        },
        "expiry": {
            "type": "user",
            "help": "Duration of quarantine.",
            "default": "5m",
        },
        "log": {
            "type": "option",
            "help": "Enable/disable AntiVirus quarantine logging.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
    },
    "content-disarm": {
        "analytics-suspicious": {
            "type": "option",
            "help": "Enable/disable using CDR as a secondary method for determining suspicous files for analytics.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "original-file-destination": {
            "type": "option",
            "help": "Destination to send original file if active content is removed.",
            "default": "discard",
            "options": ["fortisandbox", "quarantine", "discard"],
        },
        "error-action": {
            "type": "option",
            "help": "Action to be taken if CDR engine encounters an unrecoverable error.",
            "default": "log-only",
            "options": ["block", "log-only", "ignore"],
        },
        "office-macro": {
            "type": "option",
            "help": "Enable/disable stripping of macros in Microsoft Office documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "office-hylink": {
            "type": "option",
            "help": "Enable/disable stripping of hyperlinks in Microsoft Office documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "office-linked": {
            "type": "option",
            "help": "Enable/disable stripping of linked objects in Microsoft Office documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "office-embed": {
            "type": "option",
            "help": "Enable/disable stripping of embedded objects in Microsoft Office documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "office-dde": {
            "type": "option",
            "help": "Enable/disable stripping of Dynamic Data Exchange events in Microsoft Office documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "office-action": {
            "type": "option",
            "help": "Enable/disable stripping of PowerPoint action events in Microsoft Office documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-javacode": {
            "type": "option",
            "help": "Enable/disable stripping of JavaScript code in PDF documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-embedfile": {
            "type": "option",
            "help": "Enable/disable stripping of embedded files in PDF documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-hyperlink": {
            "type": "option",
            "help": "Enable/disable stripping of hyperlinks from PDF documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-act-gotor": {
            "type": "option",
            "help": "Enable/disable stripping of PDF document actions that access other PDF documents.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-act-launch": {
            "type": "option",
            "help": "Enable/disable stripping of PDF document actions that launch other applications.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-act-sound": {
            "type": "option",
            "help": "Enable/disable stripping of PDF document actions that play a sound.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-act-movie": {
            "type": "option",
            "help": "Enable/disable stripping of PDF document actions that play a movie.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-act-java": {
            "type": "option",
            "help": "Enable/disable stripping of PDF document actions that execute JavaScript code.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "pdf-act-form": {
            "type": "option",
            "help": "Enable/disable stripping of PDF document actions that submit data to other targets.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "cover-page": {
            "type": "option",
            "help": "Enable/disable inserting a cover page into the disarmed document.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "detect-only": {
            "type": "option",
            "help": "Enable/disable only detect disarmable files, do not alter content.",
            "default": "disable",
            "options": ["disable", "enable"],
        },
    },
    "external-blocklist": {
        "name": {
            "type": "string",
            "help": "External blocklist.",
            "default": "",
            "max_length": 79,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_FEATURE_SET = [
    "flow",
    "proxy",
]
VALID_BODY_FORTISANDBOX_MODE = [
    "inline",
    "analytics-suspicious",
    "analytics-everything",
]
VALID_BODY_ANALYTICS_DB = [
    "disable",
    "enable",
]
VALID_BODY_MOBILE_MALWARE_DB = [
    "disable",
    "enable",
]
VALID_BODY_OUTBREAK_PREVENTION_ARCHIVE_SCAN = [
    "disable",
    "enable",
]
VALID_BODY_EXTERNAL_BLOCKLIST_ENABLE_ALL = [
    "disable",
    "enable",
]
VALID_BODY_EMS_THREAT_FEED = [
    "disable",
    "enable",
]
VALID_BODY_FORTINDR_ERROR_ACTION = [
    "log-only",
    "block",
    "ignore",
]
VALID_BODY_FORTINDR_TIMEOUT_ACTION = [
    "log-only",
    "block",
    "ignore",
]
VALID_BODY_FORTISANDBOX_ERROR_ACTION = [
    "log-only",
    "block",
    "ignore",
]
VALID_BODY_FORTISANDBOX_TIMEOUT_ACTION = [
    "log-only",
    "block",
    "ignore",
]
VALID_BODY_AV_VIRUS_LOG = [
    "enable",
    "disable",
]
VALID_BODY_EXTENDED_LOG = [
    "enable",
    "disable",
]
VALID_BODY_SCAN_MODE = [
    "default",
    "legacy",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_antivirus_profile_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate GET request parameters for antivirus/profile.

    Args:
        attr: Attribute filter (optional)
        filters: Additional filter parameters
        **params: Other query parameters

    Returns:
        Tuple of (is_valid, error_message)

    Examples:
        >>> # Valid - Get all items
        >>> is_valid, error = validate_antivirus_profile_get()
        >>> assert is_valid == True
        
        >>> # Valid - Get specific item by name
        >>> is_valid, error = validate_antivirus_profile_get(name="test-item")
        >>> assert is_valid == True
        
        >>> # Valid - With filters
        >>> is_valid, error = validate_antivirus_profile_get(
        ...     filters={"format": "name|type"}
        ... )
        >>> assert is_valid == True
    """
    # Validate query parameters if present
    if "action" in params:
        value = params.get("action")
        if value and value not in VALID_QUERY_ACTION:
            return (
                False,
                f"Invalid query parameter 'action'='{value}'. Must be one of: {', '.join(VALID_QUERY_ACTION)}",
            )

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_required_fields(payload: dict) -> tuple[bool, str | None]:
    """
    Validate required fields for antivirus/profile.

    This validator checks:
    1. Always-required fields are present
    2. Mutually exclusive groups have at least one field

    Args:
        payload: The request payload to validate

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> payload = {"name": "test"}
        >>> is_valid, error = validate_required_fields(payload)
    """
    # Check always-required fields
    missing_fields = []
    for field in REQUIRED_FIELDS:
        if field not in payload:
            missing_fields.append(field)
    
    if missing_fields:
        # Build enhanced error message
        error_parts = [f"Missing required field(s): {', '.join(missing_fields)}"]
        
        # Add descriptions for first few missing fields
        for field in missing_fields[:3]:
            desc = FIELD_DESCRIPTIONS.get(field)
            if desc:
                error_parts.append(f"  • {field}: {desc}")
        
        if len(missing_fields) > 3:
            error_parts.append(f"  ... and {len(missing_fields) - 3} more")
        
        return (False, "\n".join(error_parts))

    return (True, None)


def validate_antivirus_profile_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate POST request to create new antivirus/profile object.

    This validator performs two-stage validation:
    1. Required fields check (schema-based)
    2. Field value validation (enums, ranges, formats)

    Args:
        payload: Request body data with configuration
        **params: Query parameters (vdom, etc.)

    Returns:
        Tuple of (is_valid, error_message)
        - is_valid: True if payload is valid, False otherwise
        - error_message: None if valid, detailed error string if invalid

    Examples:
        >>> # ✅ Valid - Minimal required fields
        >>> payload = {
        ...     "name": True,  # Profile name.
        ... }
        >>> is_valid, error = validate_antivirus_profile_post(payload)
        >>> assert is_valid == True
        
        >>> # ✅ Valid - With enum field
        >>> payload = {
        ...     "name": True,
        ...     "feature-set": "flow",  # Valid enum value
        ... }
        >>> is_valid, error = validate_antivirus_profile_post(payload)
        >>> assert is_valid == True
        
        >>> # ❌ Invalid - Wrong enum value
        >>> payload["feature-set"] = "invalid-value"
        >>> is_valid, error = validate_antivirus_profile_post(payload)
        >>> assert is_valid == False
        >>> assert "Invalid value" in error
        
        >>> # ❌ Invalid - Missing required field
        >>> payload = {}  # Empty payload
        >>> is_valid, error = validate_antivirus_profile_post(payload)
        >>> assert is_valid == False
        >>> assert "Missing required field" in error
    """
    # Step 1: Validate required fields
    is_valid, error = validate_required_fields(payload)
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values
    if "feature-set" in payload:
        value = payload["feature-set"]
        if value not in VALID_BODY_FEATURE_SET:
            desc = FIELD_DESCRIPTIONS.get("feature-set", "")
            error_msg = f"Invalid value for 'feature-set': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_FEATURE_SET)}"
            error_msg += f"\n  → Example: feature-set='{{ VALID_BODY_FEATURE_SET[0] }}'"
            return (False, error_msg)
    if "fortisandbox-mode" in payload:
        value = payload["fortisandbox-mode"]
        if value not in VALID_BODY_FORTISANDBOX_MODE:
            desc = FIELD_DESCRIPTIONS.get("fortisandbox-mode", "")
            error_msg = f"Invalid value for 'fortisandbox-mode': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_FORTISANDBOX_MODE)}"
            error_msg += f"\n  → Example: fortisandbox-mode='{{ VALID_BODY_FORTISANDBOX_MODE[0] }}'"
            return (False, error_msg)
    if "analytics-db" in payload:
        value = payload["analytics-db"]
        if value not in VALID_BODY_ANALYTICS_DB:
            desc = FIELD_DESCRIPTIONS.get("analytics-db", "")
            error_msg = f"Invalid value for 'analytics-db': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_ANALYTICS_DB)}"
            error_msg += f"\n  → Example: analytics-db='{{ VALID_BODY_ANALYTICS_DB[0] }}'"
            return (False, error_msg)
    if "mobile-malware-db" in payload:
        value = payload["mobile-malware-db"]
        if value not in VALID_BODY_MOBILE_MALWARE_DB:
            desc = FIELD_DESCRIPTIONS.get("mobile-malware-db", "")
            error_msg = f"Invalid value for 'mobile-malware-db': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_MOBILE_MALWARE_DB)}"
            error_msg += f"\n  → Example: mobile-malware-db='{{ VALID_BODY_MOBILE_MALWARE_DB[0] }}'"
            return (False, error_msg)
    if "outbreak-prevention-archive-scan" in payload:
        value = payload["outbreak-prevention-archive-scan"]
        if value not in VALID_BODY_OUTBREAK_PREVENTION_ARCHIVE_SCAN:
            desc = FIELD_DESCRIPTIONS.get("outbreak-prevention-archive-scan", "")
            error_msg = f"Invalid value for 'outbreak-prevention-archive-scan': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_OUTBREAK_PREVENTION_ARCHIVE_SCAN)}"
            error_msg += f"\n  → Example: outbreak-prevention-archive-scan='{{ VALID_BODY_OUTBREAK_PREVENTION_ARCHIVE_SCAN[0] }}'"
            return (False, error_msg)
    if "external-blocklist-enable-all" in payload:
        value = payload["external-blocklist-enable-all"]
        if value not in VALID_BODY_EXTERNAL_BLOCKLIST_ENABLE_ALL:
            desc = FIELD_DESCRIPTIONS.get("external-blocklist-enable-all", "")
            error_msg = f"Invalid value for 'external-blocklist-enable-all': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_EXTERNAL_BLOCKLIST_ENABLE_ALL)}"
            error_msg += f"\n  → Example: external-blocklist-enable-all='{{ VALID_BODY_EXTERNAL_BLOCKLIST_ENABLE_ALL[0] }}'"
            return (False, error_msg)
    if "ems-threat-feed" in payload:
        value = payload["ems-threat-feed"]
        if value not in VALID_BODY_EMS_THREAT_FEED:
            desc = FIELD_DESCRIPTIONS.get("ems-threat-feed", "")
            error_msg = f"Invalid value for 'ems-threat-feed': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_EMS_THREAT_FEED)}"
            error_msg += f"\n  → Example: ems-threat-feed='{{ VALID_BODY_EMS_THREAT_FEED[0] }}'"
            return (False, error_msg)
    if "fortindr-error-action" in payload:
        value = payload["fortindr-error-action"]
        if value not in VALID_BODY_FORTINDR_ERROR_ACTION:
            desc = FIELD_DESCRIPTIONS.get("fortindr-error-action", "")
            error_msg = f"Invalid value for 'fortindr-error-action': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_FORTINDR_ERROR_ACTION)}"
            error_msg += f"\n  → Example: fortindr-error-action='{{ VALID_BODY_FORTINDR_ERROR_ACTION[0] }}'"
            return (False, error_msg)
    if "fortindr-timeout-action" in payload:
        value = payload["fortindr-timeout-action"]
        if value not in VALID_BODY_FORTINDR_TIMEOUT_ACTION:
            desc = FIELD_DESCRIPTIONS.get("fortindr-timeout-action", "")
            error_msg = f"Invalid value for 'fortindr-timeout-action': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_FORTINDR_TIMEOUT_ACTION)}"
            error_msg += f"\n  → Example: fortindr-timeout-action='{{ VALID_BODY_FORTINDR_TIMEOUT_ACTION[0] }}'"
            return (False, error_msg)
    if "fortisandbox-error-action" in payload:
        value = payload["fortisandbox-error-action"]
        if value not in VALID_BODY_FORTISANDBOX_ERROR_ACTION:
            desc = FIELD_DESCRIPTIONS.get("fortisandbox-error-action", "")
            error_msg = f"Invalid value for 'fortisandbox-error-action': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_FORTISANDBOX_ERROR_ACTION)}"
            error_msg += f"\n  → Example: fortisandbox-error-action='{{ VALID_BODY_FORTISANDBOX_ERROR_ACTION[0] }}'"
            return (False, error_msg)
    if "fortisandbox-timeout-action" in payload:
        value = payload["fortisandbox-timeout-action"]
        if value not in VALID_BODY_FORTISANDBOX_TIMEOUT_ACTION:
            desc = FIELD_DESCRIPTIONS.get("fortisandbox-timeout-action", "")
            error_msg = f"Invalid value for 'fortisandbox-timeout-action': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_FORTISANDBOX_TIMEOUT_ACTION)}"
            error_msg += f"\n  → Example: fortisandbox-timeout-action='{{ VALID_BODY_FORTISANDBOX_TIMEOUT_ACTION[0] }}'"
            return (False, error_msg)
    if "av-virus-log" in payload:
        value = payload["av-virus-log"]
        if value not in VALID_BODY_AV_VIRUS_LOG:
            desc = FIELD_DESCRIPTIONS.get("av-virus-log", "")
            error_msg = f"Invalid value for 'av-virus-log': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_AV_VIRUS_LOG)}"
            error_msg += f"\n  → Example: av-virus-log='{{ VALID_BODY_AV_VIRUS_LOG[0] }}'"
            return (False, error_msg)
    if "extended-log" in payload:
        value = payload["extended-log"]
        if value not in VALID_BODY_EXTENDED_LOG:
            desc = FIELD_DESCRIPTIONS.get("extended-log", "")
            error_msg = f"Invalid value for 'extended-log': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_EXTENDED_LOG)}"
            error_msg += f"\n  → Example: extended-log='{{ VALID_BODY_EXTENDED_LOG[0] }}'"
            return (False, error_msg)
    if "scan-mode" in payload:
        value = payload["scan-mode"]
        if value not in VALID_BODY_SCAN_MODE:
            desc = FIELD_DESCRIPTIONS.get("scan-mode", "")
            error_msg = f"Invalid value for 'scan-mode': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SCAN_MODE)}"
            error_msg += f"\n  → Example: scan-mode='{{ VALID_BODY_SCAN_MODE[0] }}'"
            return (False, error_msg)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_antivirus_profile_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate PUT request to update antivirus/profile.

    Args:
        payload: Request body data
        **params: Query parameters

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> payload = {"name": "updated_item"}
        >>> is_valid, error = validate_antivirus_profile_put(payload)
    """
    # Step 1: Validate enum values
    if "feature-set" in payload:
        value = payload["feature-set"]
        if value not in VALID_BODY_FEATURE_SET:
            return (
                False,
                f"Invalid value for 'feature-set'='{value}'. Must be one of: {', '.join(VALID_BODY_FEATURE_SET)}",
            )
    if "fortisandbox-mode" in payload:
        value = payload["fortisandbox-mode"]
        if value not in VALID_BODY_FORTISANDBOX_MODE:
            return (
                False,
                f"Invalid value for 'fortisandbox-mode'='{value}'. Must be one of: {', '.join(VALID_BODY_FORTISANDBOX_MODE)}",
            )
    if "analytics-db" in payload:
        value = payload["analytics-db"]
        if value not in VALID_BODY_ANALYTICS_DB:
            return (
                False,
                f"Invalid value for 'analytics-db'='{value}'. Must be one of: {', '.join(VALID_BODY_ANALYTICS_DB)}",
            )
    if "mobile-malware-db" in payload:
        value = payload["mobile-malware-db"]
        if value not in VALID_BODY_MOBILE_MALWARE_DB:
            return (
                False,
                f"Invalid value for 'mobile-malware-db'='{value}'. Must be one of: {', '.join(VALID_BODY_MOBILE_MALWARE_DB)}",
            )
    if "outbreak-prevention-archive-scan" in payload:
        value = payload["outbreak-prevention-archive-scan"]
        if value not in VALID_BODY_OUTBREAK_PREVENTION_ARCHIVE_SCAN:
            return (
                False,
                f"Invalid value for 'outbreak-prevention-archive-scan'='{value}'. Must be one of: {', '.join(VALID_BODY_OUTBREAK_PREVENTION_ARCHIVE_SCAN)}",
            )
    if "external-blocklist-enable-all" in payload:
        value = payload["external-blocklist-enable-all"]
        if value not in VALID_BODY_EXTERNAL_BLOCKLIST_ENABLE_ALL:
            return (
                False,
                f"Invalid value for 'external-blocklist-enable-all'='{value}'. Must be one of: {', '.join(VALID_BODY_EXTERNAL_BLOCKLIST_ENABLE_ALL)}",
            )
    if "ems-threat-feed" in payload:
        value = payload["ems-threat-feed"]
        if value not in VALID_BODY_EMS_THREAT_FEED:
            return (
                False,
                f"Invalid value for 'ems-threat-feed'='{value}'. Must be one of: {', '.join(VALID_BODY_EMS_THREAT_FEED)}",
            )
    if "fortindr-error-action" in payload:
        value = payload["fortindr-error-action"]
        if value not in VALID_BODY_FORTINDR_ERROR_ACTION:
            return (
                False,
                f"Invalid value for 'fortindr-error-action'='{value}'. Must be one of: {', '.join(VALID_BODY_FORTINDR_ERROR_ACTION)}",
            )
    if "fortindr-timeout-action" in payload:
        value = payload["fortindr-timeout-action"]
        if value not in VALID_BODY_FORTINDR_TIMEOUT_ACTION:
            return (
                False,
                f"Invalid value for 'fortindr-timeout-action'='{value}'. Must be one of: {', '.join(VALID_BODY_FORTINDR_TIMEOUT_ACTION)}",
            )
    if "fortisandbox-error-action" in payload:
        value = payload["fortisandbox-error-action"]
        if value not in VALID_BODY_FORTISANDBOX_ERROR_ACTION:
            return (
                False,
                f"Invalid value for 'fortisandbox-error-action'='{value}'. Must be one of: {', '.join(VALID_BODY_FORTISANDBOX_ERROR_ACTION)}",
            )
    if "fortisandbox-timeout-action" in payload:
        value = payload["fortisandbox-timeout-action"]
        if value not in VALID_BODY_FORTISANDBOX_TIMEOUT_ACTION:
            return (
                False,
                f"Invalid value for 'fortisandbox-timeout-action'='{value}'. Must be one of: {', '.join(VALID_BODY_FORTISANDBOX_TIMEOUT_ACTION)}",
            )
    if "av-virus-log" in payload:
        value = payload["av-virus-log"]
        if value not in VALID_BODY_AV_VIRUS_LOG:
            return (
                False,
                f"Invalid value for 'av-virus-log'='{value}'. Must be one of: {', '.join(VALID_BODY_AV_VIRUS_LOG)}",
            )
    if "extended-log" in payload:
        value = payload["extended-log"]
        if value not in VALID_BODY_EXTENDED_LOG:
            return (
                False,
                f"Invalid value for 'extended-log'='{value}'. Must be one of: {', '.join(VALID_BODY_EXTENDED_LOG)}",
            )
    if "scan-mode" in payload:
        value = payload["scan-mode"]
        if value not in VALID_BODY_SCAN_MODE:
            return (
                False,
                f"Invalid value for 'scan-mode'='{value}'. Must be one of: {', '.join(VALID_BODY_SCAN_MODE)}",
            )

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Provide programmatic access to field metadata for IDE autocomplete,
# documentation generation, and dynamic validation
# ============================================================================


def get_field_description(field_name: str) -> str | None:
    """
    Get description/help text for a field.

    Args:
        field_name: Name of the field

    Returns:
        Description text or None if field doesn't exist

    Example:
        >>> desc = get_field_description("name")
        >>> print(desc)
    """
    return FIELD_DESCRIPTIONS.get(field_name)


def get_field_type(field_name: str) -> str | None:
    """
    Get the type of a field.

    Args:
        field_name: Name of the field

    Returns:
        Field type (e.g., "string", "integer", "option") or None

    Example:
        >>> field_type = get_field_type("status")
        >>> print(field_type)  # "option"
    """
    return FIELD_TYPES.get(field_name)


def get_field_constraints(field_name: str) -> dict[str, Any] | None:
    """
    Get constraints for a field (min/max values, string length).

    Args:
        field_name: Name of the field

    Returns:
        Constraint dict or None

    Example:
        >>> constraints = get_field_constraints("port")
        >>> print(constraints)  # {"type": "integer", "min": 1, "max": 65535}
    """
    return FIELD_CONSTRAINTS.get(field_name)


def get_field_default(field_name: str) -> Any | None:
    """
    Get default value for a field.

    Args:
        field_name: Name of the field

    Returns:
        Default value or None if no default

    Example:
        >>> default = get_field_default("status")
        >>> print(default)  # "enable"
    """
    return FIELDS_WITH_DEFAULTS.get(field_name)


def get_field_options(field_name: str) -> list[str] | None:
    """
    Get valid enum options for a field.

    Args:
        field_name: Name of the field

    Returns:
        List of valid values or None if not an enum field

    Example:
        >>> options = get_field_options("status")
        >>> print(options)  # ["enable", "disable"]
    """
    # Construct the constant name from field name
    # Replace all non-alphanumeric characters with underscores for valid Python identifiers
    import re
    safe_name = re.sub(r'[^a-zA-Z0-9]', '_', field_name)
    constant_name = f"VALID_BODY_{safe_name.upper()}"
    return globals().get(constant_name)


def get_nested_schema(field_name: str) -> dict[str, Any] | None:
    """
    Get schema for nested table/list fields.

    Args:
        field_name: Name of the parent field

    Returns:
        Dict mapping child field names to their metadata

    Example:
        >>> nested = get_nested_schema("members")
        >>> if nested:
        ...     for child_field, child_meta in nested.items():
        ...         print(f"{child_field}: {child_meta['type']}")
    """
    return NESTED_SCHEMAS.get(field_name)


def get_all_fields() -> list[str]:
    """
    Get list of all field names.

    Returns:
        List of all field names in the schema

    Example:
        >>> fields = get_all_fields()
        >>> print(len(fields))
    """
    return list(FIELD_TYPES.keys())


def get_field_metadata(field_name: str) -> dict[str, Any] | None:
    """
    Get complete metadata for a field (type, description, constraints, defaults, options).

    Args:
        field_name: Name of the field

    Returns:
        Dict with all available metadata or None if field doesn't exist

    Example:
        >>> meta = get_field_metadata("status")
        >>> print(meta)
        >>> # {
        >>> #   "type": "option",
        >>> #   "description": "Enable/disable this feature",
        >>> #   "default": "enable",
        >>> #   "options": ["enable", "disable"]
        >>> # }
    """
    if field_name not in FIELD_TYPES:
        return None

    metadata = {
        "name": field_name,
        "type": FIELD_TYPES[field_name],
    }

    # Add description if available
    if field_name in FIELD_DESCRIPTIONS:
        metadata["description"] = FIELD_DESCRIPTIONS[field_name]

    # Add constraints if available
    if field_name in FIELD_CONSTRAINTS:
        metadata["constraints"] = FIELD_CONSTRAINTS[field_name]

    # Add default if available
    if field_name in FIELDS_WITH_DEFAULTS:
        metadata["default"] = FIELDS_WITH_DEFAULTS[field_name]

    # Add required flag
    metadata["required"] = field_name in REQUIRED_FIELDS

    # Add options if available
    options = get_field_options(field_name)
    if options:
        metadata["options"] = options

    # Add nested schema if available
    nested = get_nested_schema(field_name)
    if nested:
        metadata["nested_schema"] = nested

    return metadata


def validate_field_value(field_name: str, value: Any) -> tuple[bool, str | None]:
    """
    Validate a single field value against its constraints.

    Args:
        field_name: Name of the field
        value: Value to validate

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> is_valid, error = validate_field_value("status", "enable")
        >>> if not is_valid:
        ...     print(error)
    """
    # Get field metadata
    field_type = get_field_type(field_name)
    if field_type is None:
        return (False, f"Unknown field: '{field_name}' (not defined in schema)")

    # Get field description for better error context
    description = get_field_description(field_name)

    # Validate enum values
    options = get_field_options(field_name)
    if options and value not in options:
        error_msg = f"Invalid value for '{field_name}': {repr(value)}"
        if description:
            error_msg += f"\n  → Description: {description}"
        error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in options)}"
        if options:
            error_msg += f"\n  → Example: {field_name}={repr(options[0])}"
        return (False, error_msg)

    # Validate constraints
    constraints = get_field_constraints(field_name)
    if constraints:
        constraint_type = constraints.get("type")

        if constraint_type == "integer":
            if not isinstance(value, int):
                error_msg = f"Field '{field_name}' must be an integer"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → You provided: {type(value).__name__} = {repr(value)}"
                return (False, error_msg)

            min_val = constraints.get("min")
            max_val = constraints.get("max")

            if min_val is not None and value < min_val:
                error_msg = f"Field '{field_name}' value {value} is below minimum {min_val}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                if max_val is not None:
                    error_msg += f"\n  → Valid range: {min_val} to {max_val}"
                return (False, error_msg)

            if max_val is not None and value > max_val:
                error_msg = f"Field '{field_name}' value {value} exceeds maximum {max_val}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                if min_val is not None:
                    error_msg += f"\n  → Valid range: {min_val} to {max_val}"
                return (False, error_msg)

        elif constraint_type == "string":
            if not isinstance(value, str):
                error_msg = f"Field '{field_name}' must be a string"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → You provided: {type(value).__name__} = {repr(value)}"
                return (False, error_msg)

            max_length = constraints.get("max_length")
            if max_length and len(value) > max_length:
                error_msg = f"Field '{field_name}' length {len(value)} exceeds maximum {max_length}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → Your value: {repr(value[:50])}{'...' if len(value) > 50 else ''}"
                return (False, error_msg)

    return (True, None)


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "antivirus/profile",
    "category": "cmdb",
    "api_path": "antivirus/profile",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure AntiVirus profiles.",
    "total_fields": 32,
    "required_fields_count": 1,
    "fields_with_defaults_count": 19,
}


def get_schema_info() -> dict[str, Any]:
    """
    Get information about this endpoint schema.

    Returns:
        Dict with schema metadata

    Example:
        >>> info = get_schema_info()
        >>> print(f"Endpoint: {info['endpoint']}")
        >>> print(f"Total fields: {info['total_fields']}")
    """
    return SCHEMA_INFO.copy()



from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class SettingPayload(TypedDict, total=False):
    """
    Type hints for alertemail/setting payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: SettingPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    username: NotRequired[str]  # Name that appears in the From: field of alert emails (max. 6
    mailto1: NotRequired[str]  # Email address to send alert email to (usually a system admin
    mailto2: NotRequired[str]  # Optional second email address to send alert email to (max. 6
    mailto3: NotRequired[str]  # Optional third email address to send alert email to (max. 63
    filter_mode: NotRequired[Literal["category", "threshold"]]  # How to filter log messages that are sent to alert emails.
    email_interval: NotRequired[int]  # Interval between sending alert emails (1 - 99999 min, defaul
    IPS_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable IPS logs in alert email.
    firewall_authentication_failure_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable firewall authentication failure logs in alert
    HA_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable HA logs in alert email.
    IPsec_errors_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable IPsec error logs in alert email.
    FDS_update_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable FortiGuard update logs in alert email.
    PPP_errors_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable PPP error logs in alert email.
    sslvpn_authentication_errors_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable Agentless VPN authentication error logs in al
    antivirus_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable antivirus logs in alert email.
    webfilter_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable web filter logs in alert email.
    configuration_changes_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable configuration change logs in alert email.
    violation_traffic_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable violation traffic logs in alert email.
    admin_login_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable administrator login/logout logs in alert emai
    FDS_license_expiring_warning: NotRequired[Literal["enable", "disable"]]  # Enable/disable FortiGuard license expiration warnings in ale
    log_disk_usage_warning: NotRequired[Literal["enable", "disable"]]  # Enable/disable disk usage warnings in alert email.
    fortiguard_log_quota_warning: NotRequired[Literal["enable", "disable"]]  # Enable/disable FortiCloud log quota warnings in alert email.
    amc_interface_bypass_mode: NotRequired[Literal["enable", "disable"]]  # Enable/disable Fortinet Advanced Mezzanine Card (AMC) interf
    FIPS_CC_errors: NotRequired[Literal["enable", "disable"]]  # Enable/disable FIPS and Common Criteria error logs in alert 
    FSSO_disconnect_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable logging of FSSO collector agent disconnect.
    ssh_logs: NotRequired[Literal["enable", "disable"]]  # Enable/disable SSH logs in alert email.
    local_disk_usage: NotRequired[int]  # Disk usage percentage at which to send alert email (1 - 99 p
    emergency_interval: NotRequired[int]  # Emergency alert interval in minutes.
    alert_interval: NotRequired[int]  # Alert alert interval in minutes.
    critical_interval: NotRequired[int]  # Critical alert interval in minutes.
    error_interval: NotRequired[int]  # Error alert interval in minutes.
    warning_interval: NotRequired[int]  # Warning alert interval in minutes.
    notification_interval: NotRequired[int]  # Notification alert interval in minutes.
    information_interval: NotRequired[int]  # Information alert interval in minutes.
    debug_interval: NotRequired[int]  # Debug alert interval in minutes.
    severity: NotRequired[Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]]  # Lowest severity level to log.


class Setting:
    """
    Configure alert email settings.
    
    Path: alertemail/setting
    Category: cmdb
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: SettingPayload | None = ...,
        username: str | None = ...,
        mailto1: str | None = ...,
        mailto2: str | None = ...,
        mailto3: str | None = ...,
        filter_mode: Literal["category", "threshold"] | None = ...,
        email_interval: int | None = ...,
        IPS_logs: Literal["enable", "disable"] | None = ...,
        firewall_authentication_failure_logs: Literal["enable", "disable"] | None = ...,
        HA_logs: Literal["enable", "disable"] | None = ...,
        IPsec_errors_logs: Literal["enable", "disable"] | None = ...,
        FDS_update_logs: Literal["enable", "disable"] | None = ...,
        PPP_errors_logs: Literal["enable", "disable"] | None = ...,
        sslvpn_authentication_errors_logs: Literal["enable", "disable"] | None = ...,
        antivirus_logs: Literal["enable", "disable"] | None = ...,
        webfilter_logs: Literal["enable", "disable"] | None = ...,
        configuration_changes_logs: Literal["enable", "disable"] | None = ...,
        violation_traffic_logs: Literal["enable", "disable"] | None = ...,
        admin_login_logs: Literal["enable", "disable"] | None = ...,
        FDS_license_expiring_warning: Literal["enable", "disable"] | None = ...,
        log_disk_usage_warning: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        username: str | None = ...,
        mailto1: str | None = ...,
        mailto2: str | None = ...,
        mailto3: str | None = ...,
        filter_mode: Literal["category", "threshold"] | None = ...,
        email_interval: int | None = ...,
        IPS_logs: Literal["enable", "disable"] | None = ...,
        firewall_authentication_failure_logs: Literal["enable", "disable"] | None = ...,
        HA_logs: Literal["enable", "disable"] | None = ...,
        IPsec_errors_logs: Literal["enable", "disable"] | None = ...,
        FDS_update_logs: Literal["enable", "disable"] | None = ...,
        PPP_errors_logs: Literal["enable", "disable"] | None = ...,
        sslvpn_authentication_errors_logs: Literal["enable", "disable"] | None = ...,
        antivirus_logs: Literal["enable", "disable"] | None = ...,
        webfilter_logs: Literal["enable", "disable"] | None = ...,
        configuration_changes_logs: Literal["enable", "disable"] | None = ...,
        violation_traffic_logs: Literal["enable", "disable"] | None = ...,
        admin_login_logs: Literal["enable", "disable"] | None = ...,
        FDS_license_expiring_warning: Literal["enable", "disable"] | None = ...,
        log_disk_usage_warning: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: SettingPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Setting",
    "SettingPayload",
]